
///////////////////////////////////////////////////////////////////////////////
// String

String.format = function(strFormat) {
   var result = '';
   var format = arguments[0];
   for(var i = 0;;) {
      var open = format.indexOf('{', i);
      var close = format.indexOf('}', i);
      if((open < 0) && (close < 0)) {
         result += format.slice(i);
         break;
      }
      if((close > 0) && ((close < open) || (open < 0))) {
         if(format.charAt(close + 1) !== '}') {
            throw new Error(600, 'String format brace mismatch');
         }
         result += format.slice(i, close + 1);
         i = close + 2;
         continue;
      }
      result += format.slice(i, open);
      i = open + 1;
      if(format.charAt(i) === '{') {
         result += '{';
         i++;
         continue;
      }
      if(close < 0) {
         throw new Error(600, 'String format brace mismatch');
      }
      var brace = format.substring(i, close);
      var colonIndex = brace.indexOf(':');
      var argNumber = parseInt((colonIndex < 0) ? brace : brace.substring(0, colonIndex), 10) + 1;
      if(isNaN(argNumber)) {
         throw new Error(600, 'Invalid string format');
      }
      var argFormat = (colonIndex < 0) ? '' : brace.substring(colonIndex + 1);
      var arg = arguments[argNumber];
      if(typeof(arg) === 'undefined' || arg === null) {
         arg = '';
      }
      if(arg.toFormattedString) {
         result += arg.toFormattedString(argFormat);
      }
      else if(arg.format) {
         result += arg.format(argFormat);
      }
      else {
         result += arg.toString();
      }
      i = close + 1;
   }
   return result;
}

///////////////////////////////////////////////////////////////////////////////
// Boolean

Boolean.parse = function(varValue) {
   if(typeof(varValue) === 'string') {
      if(varValue.toLowerCase() === 'true') {
         return true;
      }
      else if(varValue.toLowerCase() === 'false' || varValue.length == 0) {
         return false;
      }
   }
   else if(typeof(varValue) === 'undefined' || varValue == null) {
      return false;
   }
   else if(typeof(varValue) === 'boolean') {
      return varValue;
   }

   throw new Error(600, String.format('Value \'{0}\' cannot be evaluated as boolean', varValue));
};

///////////////////////////////////////////////////////////////////////////////
// Date

Date.DateRange = function() {
   this.First = 0;
   this.Last = 0;
};

Date.GetDateRange = function(nMonth, nFullYear) {
   var objDateRange;
   
   var bLeapYear = (((nFullYear % 4) == 0) && ((nFullYear % 100) != 0) || ((nFullYear % 400) == 0));
   
   if(nMonth == 1 || nMonth == 3 || nMonth == 5 || nMonth == 7 || nMonth == 8 || nMonth == 10 || nMonth == 12) {
      objDateRange = new Date.DateRange();
      objDateRange.First = 1;
      objDateRange.Last = 31;
   }
   else if(nMonth == 4 || nMonth == 6 || nMonth == 9 || nMonth == 11) {
      objDateRange = new Date.DateRange();
      objDateRange.First = 1;
      objDateRange.Last = 30;
   }
   else if(nMonth == 2) {
      objDateRange = new Date.DateRange();
      objDateRange.First = 1;
      if(bLeapYear) {
         objDateRange.Last = 29;
      }
      else {
         objDateRange.Last = 28;
      }
   }
   else {
      throw new Error(600, String.format('Month is expected to be in the range 0 - 11. Actual value is \'{0}\'', nMonth));
   }
   
   return objDateRange;
}



var MgScopes = new Object();

///////////////////////////////////////////////////////////////////////////////
// Debug

MgScopes.Debug = function() {
};

MgScopes.Debug.LOG_PATH = 'C:\\Users\\Alex\\AppData\\Local\\Temp\\Lifetime.log';

MgScopes.Debug.WriteLine = function(strMessage) {
   var objFSO = new ActiveXObject('Scripting.FileSystemObject');
   var objFile = objFSO.OpenTextFile(MgScopes.Debug.LOG_PATH, 8, true);
   objFile.WriteLine(strMessage);
   objFile.Close();
};

///////////////////////////////////////////////////////////////////////////////
// SettingsModel

MgScopes.SettingsModel = function() {
   this._StartDateTime = null;
   this._TargetDateTime = null;
   this._SleepHourPerDay = 0;
   
   this.Initialize();
};

MgScopes.SettingsModel.SLEEP_HOUR_PER_DAY = 8;

MgScopes.SettingsModel.SECOND_SPAN = 1000;
MgScopes.SettingsModel.MINUTE_SPAN = MgScopes.SettingsModel.SECOND_SPAN * 60;
MgScopes.SettingsModel.HOUR_SPAN = MgScopes.SettingsModel.MINUTE_SPAN * 60;
MgScopes.SettingsModel.DAY_SPAN = MgScopes.SettingsModel.HOUR_SPAN * 24;
MgScopes.SettingsModel.WEEK_SPAN = MgScopes.SettingsModel.DAY_SPAN * 7;

MgScopes.SettingsModel.prototype = {
   Initialize: function() {
      var objCurrentDateTime = new Date();
      
      this._StartDateTime = new Date(objCurrentDateTime.getTime() - MgScopes.SettingsModel.DAY_SPAN * 365 * 20);
      this._TargetDateTime = new Date(objCurrentDateTime.getTime() + MgScopes.SettingsModel.DAY_SPAN * 365 * 20);
      this._SleepHourPerDay = MgScopes.SettingsModel.SLEEP_HOUR_PER_DAY;
   },

   Dispose: function() {
   },
   
   Load: function() {
      if(Boolean.parse(System.Gadget.Settings.readString('IsSaved'))) {
         this._StartDateTime = new Date(Date.parse(System.Gadget.Settings.readString('StartDateTime')));
         this._TargetDateTime = new Date(Date.parse(System.Gadget.Settings.readString('TargetDateTime')));
         this._SleepHourPerDay = parseInt(System.Gadget.Settings.readString('SleepHourPerDay'));
      }
   },
   
   Save: function() {
      System.Gadget.Settings.writeString('IsSaved', true.toString());

      System.Gadget.Settings.writeString('StartDateTime', this._StartDateTime.toString());
      System.Gadget.Settings.writeString('TargetDateTime', this._TargetDateTime.toString());
      System.Gadget.Settings.writeString('SleepHourPerDay', this._SleepHourPerDay.toString());
   },
   
   _GetDateString: function(objDateTime) {
      var nMonth = objDateTime.getMonth() + 1;
      var nDate = objDateTime.getDate();
      var nFullYear = objDateTime.getFullYear();
      return (
         ((nMonth < 10) ? '0' : '') +
         nMonth.toString() +
         '/' +
         ((nDate < 10) ? '0' : '') +
         nDate.toString() +
         '/' +
         nFullYear.toString()
      );
   },
   
   _SetDateString: function(strDate, fnGetter, fnSetter) {
      var strarrDate = strDate.split('/');
      if(strarrDate.length != 3) {
         throw new Error(600, 'Invalid date format. Expected MM/dd/yyyy');
      }

      var nMonth = parseInt(strarrDate[0]);
      if(nMonth < 1 || nMonth > 12) {
         throw new Error(600, String.format('Month is expected to be in the range 1 - 12. Actual value is \'{0}\'', nMonth));
      }
      var nFullYear = parseInt(strarrDate[2]);
      if(nFullYear < 0 || nFullYear > 9999) {
         throw new Error(600, String.format('Year is expected to be in the range 0 - 9999. Actual value is \'{0}\'', nFullYear));
      }
      var nDate = parseInt(strarrDate[1]);
      var objDateRange = Date.GetDateRange(nMonth, nFullYear);
      if(nDate < objDateRange.First || nDate > objDateRange.Last) {
         throw new Error(600, String.format('Date is expected to be in the range \'{0}\' - \'{1}\'. Actual value is \'{2}\'', objDateRange.First, objDateRange.Last, nDate));
      }

      nMonth--;

      if(fnGetter() == null) {
         fnSetter(new Date(nFullYear, nMonth, nDate));
      }
      else {
         fnGetter().setFullYear(nFullYear);
         fnGetter().setMonth(nMonth);
         fnGetter().setDate(nDate);
      }
   },
   
   _GetTimeString: function(objDateTime) {
      var nHours = objDateTime.getHours();
      var nMinutes = objDateTime.getMinutes();
      var nSeconds = objDateTime.getSeconds();
      return (
         ((nHours < 10) ? '0' : '') +
         nHours.toString() +
         ':' +
         ((nMinutes < 10) ? '0' : '') +
         nMinutes.toString() +
         ':' +
         ((nSeconds < 10) ? '0' : '') +
         nSeconds.toString()
      );
   },
   
   _SetTimeString: function(strTime, fnGetter, fnSetter) {
      var strarrTime = strTime.split(':');
      if(strarrTime.length > 3 || strarrTime.length < 2) {
         throw new Error(600, 'Invalid time format. Expected hh:mm:ss or hh:mm');
      }

      var nHours = parseInt(strarrTime[0]);
      if(nHours < 0 || nHours > 23) {
         throw new Error(600, String.format('Hours is expected to be in the range 0 - 23. Actual value is \'{0}\'', nHours));
      }
      var nMinutes = parseInt(strarrTime[1]);
      if(nMinutes < 0 || nMinutes > 59) {
         throw new Error(600, String.format('Hours is expected to be in the range 0 - 59. Actual value is \'{0}\'', nMinutes));
      }
      var nSeconds;
      if(strarrTime.length == 3) {
         nSeconds = parseInt(strarrTime[2]);
      }
      else {
         nSeconds = 0;
      }
      if(nSeconds < 0 || nSeconds > 59) {
         throw new Error(600, String.format('Hours is expected to be in the range 0 - 59. Actual value is \'{0}\'', nSeconds));
      }

      if(fnGetter() == null) {
         var objNow = new Date();
         fnSetter(new Date(objNow.getFullYear(), objNow.getMonth(), objNow.getDate(), nHours, nMinutes, nSeconds));
      }
      else {
         fnGetter().setHours(nHours);
         fnGetter().setMinutes(nMinutes);
         fnGetter().setSeconds(nSeconds);
      }
   },
   
   get_StartDateTime: function() {
      return this._StartDateTime;
   },
   set_StartDateTime: function(objValue) {
      this._StartDateTime = objValue;
   },

   get_StartDateString: function() {
      return this._GetDateString(this._StartDateTime);
   },
   set_StartDateString: function(strValue) {
      var objThis = this;
      this._SetDateString(
         strValue,
         function() {
            return objThis._StartDateTime;
         },
         function(lobj) {
            objThis._StartDateTime = lobj;
         }
      );
   },
   
   get_StartTimeString: function() {
      return this._GetTimeString(this._StartDateTime);
   },
   set_StartTimeString: function(strValue) {
      var objThis = this;
      this._SetTimeString(
         strValue,
         function() {
            return objThis._StartDateTime;
         },
         function(lobj) {
            objThis._StartDateTime = lobj;
         }
      );
   },

   get_TargetDateTime: function() {
      return this._TargetDateTime;
   },
   set_TargetDateTime: function(objValue) {
      this._TargetDateTime = objValue;
   },

   get_TargetDateString: function() {
      return this._GetDateString(this._TargetDateTime);
   },
   set_TargetDateString: function(strValue) {
      var objThis = this;
      this._SetDateString(
         strValue,
         function() {
            return objThis._TargetDateTime;
         },
         function(lobj) {
            objThis._TargetDateTime = lobj;
         }
      );
   },
   
   get_TargetTimeString: function() {
      return this._GetTimeString(this._TargetDateTime);
   },
   set_TargetTimeString: function(strValue) {
      var objThis = this;
      this._SetTimeString(
         strValue,
         function() {
            return objThis._TargetDateTime;
         },
         function(lobj) {
            objThis._TargetDateTime = lobj;
         }
      );
   },

   get_SleepHourPerDay: function() {
      return this._SleepHourPerDay;
   },
   set_SleepHourPerDay: function(nValue) {
      if(typeof(nValue) !== 'number' || isNaN(nValue)) {
         throw new Error(600, String.format('Sleep hours per day must be a number. Actual \'{0}\'', nValue));
      }
      if(nValue < 0 || nValue > 24) {
         throw new Error(600, String.format('Sleep hours per day is expected to be in the range 0 - 24. Actual value is \'{0}\'', nValue));
      }
      this._SleepHourPerDay = nValue;
   }
};

///////////////////////////////////////////////////////////////////////////////
// Lifetime

MgScopes.Lifetime = function(objSettingsModel) {
   this._SettingsModel = objSettingsModel;
   
   this.Initialize();
};

MgScopes.Lifetime.prototype = {
   Initialize: function() {
   },

   Dispose: function() {
   },

   GetPassedDayCount: function() {
      if(this._SettingsModel.get_StartDateTime() == null) {
         throw new Error(600, 'StartDateTime must not be null');
      }
      var iMillisecondDiff = Date.now() - this._SettingsModel.get_StartDateTime().getTime();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.DAY_SPAN);
   },

   GetPassedHourCount: function() {
      if(this._SettingsModel.get_StartDateTime() == null) {
         throw new Error(600, 'StartDateTime must not be null');
      }
      var iMillisecondDiff = Date.now() - this._SettingsModel.get_StartDateTime().getTime();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.HOUR_SPAN);
   },

   GetPassedEffectiveHourCount: function() {
      if(this._SettingsModel.get_StartDateTime() == null) {
         throw new Error(600, 'StartDateTime must not be null');
      }
      var iMillisecondDiff = Date.now() - this._SettingsModel.get_StartDateTime().getTime();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.DAY_SPAN * (24 - this._SettingsModel.get_SleepHourPerDay()));
   },

   GetLeftDayCount: function() {
      if(this._SettingsModel.get_TargetDateTime() == null) {
         throw new Error(600, 'TargetDateTime must not be null');
      }
      var iMillisecondDiff = this._SettingsModel.get_TargetDateTime().getTime() - Date.now();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.DAY_SPAN);
   },

   GetLeftHourCount: function() {
      if(this._SettingsModel.get_TargetDateTime() == null) {
         throw new Error(600, 'TargetDateTime must not be null');
      }
      var iMillisecondDiff = this._SettingsModel.get_TargetDateTime().getTime() - Date.now();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.HOUR_SPAN);
   },

   GetLeftEffectiveHourCount: function() {
      if(this._SettingsModel.get_TargetDateTime() == null) {
         throw new Error(600, 'TargetDateTime must not be null');
      }
      var iMillisecondDiff = this._SettingsModel.get_TargetDateTime().getTime() - Date.now();
      return Math.floor(iMillisecondDiff / MgScopes.SettingsModel.DAY_SPAN * (24 - this._SettingsModel.get_SleepHourPerDay()));
   }
};

///////////////////////////////////////////////////////////////////////////////
// Gadget

MgScopes.Gadget = function(objSettingsModel) {
   this._SettingsModel = objSettingsModel;

   this._Lifetime = null;

   this._Body = null;
   this._MainContent = null;
   
   this._DayPassedLabel = null;
   this._HourPassedLabel = null;
   this._EffectiveHourPassedLabel = null;

   this._DayLeftLabel = null;
   this._HourLeftLabel = null;
   this._EffectiveHourLeftLabel = null;

   this._InfoPanel = null;
   
   this.Initialize();
};

MgScopes.Gadget.prototype = {
   Initialize: function() {
      this._SettingsModel.Load();
      this._Lifetime = new MgScopes.Lifetime(this._SettingsModel);

      var objThis = this;
      System.Gadget.onSettingsClosed = function() {
         objThis._SettingsClosed.apply(objThis, arguments);
      };
   },

   Dispose: function() {
   },

   Load: function() {
      this._Body = $('#Body');
      this._MainContent = $('#MainContent');
   
      this._DayPassedLabel = $('#lblDayPassed');
      this._HourPassedLabel = $('#lblHourPassed');
      this._EffectiveHourPassedLabel = $('#lblEffectiveHourPassed');

      this._DayLeftLabel = $('#lblDayLeft');
      this._HourLeftLabel = $('#lblHourLeft');
      this._EffectiveHourLeftLabel = $('#lblEffectiveHourLeft');
      
      this._InfoPanel = $('#InfoPanel');
      
      var objThis = this;
      window.setInterval(function() {
         objThis._Update.apply(objThis, arguments);
      }, 60 * 1000);
      window.setTimeout(function() {
         objThis._ShowInfo.apply(objThis, arguments);
      }, 15 * 60 * 1000);
      
      this._Update();
   },
   
   _SettingsClosed: function(objEvent) {
      if(objEvent.closeAction == objEvent.Action.commit) {
         this._Update();
      }
   },

   _Update: function() {
      this._DayPassedLabel.text(this._Lifetime.GetPassedDayCount());
      this._HourPassedLabel.text(this._Lifetime.GetPassedHourCount());
      this._EffectiveHourPassedLabel.text(this._Lifetime.GetPassedEffectiveHourCount());

      this._DayLeftLabel.text(this._Lifetime.GetLeftDayCount());
      this._HourLeftLabel.text(this._Lifetime.GetLeftHourCount());
      this._EffectiveHourLeftLabel.text(this._Lifetime.GetLeftEffectiveHourCount());
   },
   
   _ShowInfo: function() {
      this._InfoPanel.html('<div class="info"><a href="http://www.megascopes.com/?utm_campaign=sft&utm_medium=post" class="info_text">&copy; MegaScopes.com</a></div>');
      this._Body.animate({ height: '145px'}, 500);
      this._MainContent.animate({ height: '141px'}, 500);
   }
};

///////////////////////////////////////////////////////////////////////////////
// Settings

MgScopes.Settings = function(objSettingsModel) {
   this._SettingsModel = objSettingsModel;

   this._StartDateLabel = null;
   this._StartTimeLabel = null;
   this._TargetDateLabel = null;
   this._TargetTimeLabel = null;
   this._SleepHourPerDayLabel = null;
   
   this._StartDateCtrl = null;
   this._StartTimeCtrl = null;
   this._TargetDateCtrl = null;
   this._TargetTimeCtrl = null;
   this._SleepHourPerDayCtrl = null;
   
   this.Initialize();
};

MgScopes.Settings.prototype = {
   Initialize: function() {
      var objThis = this;
      System.Gadget.onSettingsClosing = function() {
         objThis._SettingsClosing.apply(objThis, arguments);
      };
   },

   Dispose: function() {
   },

   Load: function() {
      this._StartDateLabel = $('#lblStartDate');
      this._StartTimeLabel = $('#lblStartTime');
      this._TargetDateLabel = $('#lblTargetDate');
      this._TargetTimeLabel = $('#lblTargetTime');
      this._SleepHourPerDayLabel = $('#lblSleepHourPerDay');

      this._StartDateCtrl = $('#txtStartDate');
      this._StartTimeCtrl = $('#txtStartTime');
      this._TargetDateCtrl = $('#txtTargetDate');
      this._TargetTimeCtrl = $('#txtTargetTime');
      this._SleepHourPerDayCtrl = $('#txtSleepHourPerDay');
      
      this._UpdateView();
   },

   _SettingsClosing: function(objEvent) {
      if(objEvent.closeAction == objEvent.Action.commit) {
         if(this._UpdateModel()) {
            this._SettingsModel.Save();
         }
         else {
            objEvent.cancel = true;
         }
      }
   },

   _UpdateView: function() {
      this._StartDateCtrl.val(this._SettingsModel.get_StartDateString());
      this._StartTimeCtrl.val(this._SettingsModel.get_StartTimeString());
      this._TargetDateCtrl.val(this._SettingsModel.get_TargetDateString());
      this._TargetTimeCtrl.val(this._SettingsModel.get_TargetTimeString());
      this._SleepHourPerDayCtrl.val(this._SettingsModel.get_SleepHourPerDay().toString());
   },
   
   _UpdateModel: function() {
      var bRes = true;

      try {
         this._SettingsModel.set_StartDateString(this._StartDateCtrl.val());
         this._StartDateLabel.removeClass('error_text').addClass('normal_text');
      }
      catch(objError) {
         bRes = false;
         this._StartDateLabel.removeClass('normal_text').addClass('error_text');
      }
      
      try {
         this._SettingsModel.set_StartTimeString(this._StartTimeCtrl.val());
         this._StartTimeLabel.removeClass('error_text').addClass('normal_text');
      }
      catch(objError) {
         bRes = false;
         this._StartTimeLabel.removeClass('normal_text').addClass('error_text');
      }
      
      try {
         this._SettingsModel.set_TargetDateString(this._TargetDateCtrl.val());
         this._TargetDateLabel.removeClass('error_text').addClass('normal_text');
      }
      catch(objError) {
         bRes = false;
         this._TargetDateLabel.removeClass('normal_text').addClass('error_text');
      }
      
      try {
         this._SettingsModel.set_TargetTimeString(this._TargetTimeCtrl.val());
         this._TargetTimeLabel.removeClass('error_text').addClass('normal_text');
      }
      catch(objError) {
         bRes = false;
         this._TargetTimeLabel.removeClass('normal_text').addClass('error_text');
      }
      
      try {
         this._SettingsModel.set_SleepHourPerDay(parseInt(this._SleepHourPerDayCtrl.val()));
         this._SleepHourPerDayLabel.removeClass('error_text').addClass('normal_text');
      }
      catch(objError) {
         bRes = false;
         this._SleepHourPerDayLabel.removeClass('normal_text').addClass('error_text');
      }
      
      return bRes;
   }
};
